package models

import (
	"encoding/json"
	"log"
	"net/http"

	"forge.grandlyon.com/web-et-numerique/factory/llle_project/backoffice-server/internal/common"
	"forge.grandlyon.com/web-et-numerique/factory/llle_project/backoffice-server/internal/constants"
)

type Poll struct {
	Year     int    `json:"year"`
	Month    int    `json:"month"`
	Question string `json:"question"`
	Link     string `json:"link"`
}

func (dh *DataHandler) GetAllPolls(w http.ResponseWriter, r *http.Request) {
	var polls []Poll
	err := dh.sqlClient.Find(&polls).Error
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	w.Header().Set(constants.ContentType, constants.Json)
	json.NewEncoder(w).Encode(polls)
	log.Printf("| get all polls | %v", r.RemoteAddr)
}

func (dh *DataHandler) GetSinglePoll(w http.ResponseWriter, r *http.Request) {
	year, month, err := common.YearMonthFromRequest(r)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	w.Header().Set(constants.ContentType, constants.Json)

	var poll Poll
	err = dh.sqlClient.Where("year = ? AND month = ?", year, month).First(&poll).Error
	if err != nil {
		// If not found, answer "not found"
		w.WriteHeader(http.StatusNotFound)
		return
	}
	json.NewEncoder(w).Encode(poll)
}

func (dh *DataHandler) SavePoll(w http.ResponseWriter, r *http.Request) {
	if r.Body == http.NoBody {
		http.Error(w, "request body is empty", http.StatusBadRequest)
		return
	}

	decoder := json.NewDecoder(r.Body)
	var poll Poll
	err := decoder.Decode(&poll)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	// Check if this poll exists
	err = dh.sqlClient.Where("year = ? AND month = ?", poll.Year, poll.Month).First(&Poll{}).Error

	if err != nil {
		// Create a poll
		err = dh.sqlClient.Create(&Poll{Year: poll.Year, Month: poll.Month, Question: poll.Question, Link: poll.Link}).Error
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			return
		}
		w.WriteHeader(http.StatusCreated)
		json.NewEncoder(w).Encode(poll)
		log.Printf("| new poll | year : %d month : %d | %v", poll.Year, poll.Month, r.RemoteAddr)
		return

	} else {
		// Update question
		err = dh.sqlClient.Model(&Poll{}).Where("year = ? AND month = ?", poll.Year, poll.Month).Update("question", poll.Question).Error
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			return
		}
		// Update link
		err = dh.sqlClient.Model(&Poll{}).Where("year = ? AND month = ?", poll.Year, poll.Month).Update("link", poll.Link).Error
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		json.NewEncoder(w).Encode(poll)
		log.Printf("| updated poll year : %d month : %d | %v", poll.Year, poll.Month, r.RemoteAddr)
	}
}

func (dh *DataHandler) DeletePoll(w http.ResponseWriter, r *http.Request) {
	year, month, err := common.YearMonthFromRequest(r)
	if err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}

	err = dh.sqlClient.Where("year = ? AND month = ?", year, month).Delete(&Poll{}).Error
	if err != nil {
		http.Error(w, err.Error(), http.StatusNotFound)
		return
	}

	w.WriteHeader(http.StatusOK)
	w.Write([]byte("successful delete"))
}
